@@ -1,3 +1,143 @@
+package DBIx::Struct::JSON::Array;
+use strict;
+use warnings;
+
+sub TIEARRAY {
+	bless [$_[1], $_[2], $_[3]], $_[0];
+}
+
+sub FETCHSIZE {
+	scalar @{$_[0][0]};
+}
+
+sub STORESIZE {
+	$_[0][1]{$_[0][2]} = undef;
+	$#{$_[0][0]} = $_[1] - 1;
+}
+
+sub STORE {
+	$_[0][1]{$_[0][2]} = undef;
+	$_[0][0][$_[1]] = $_[2];
+}
+sub FETCH {$_[0][0][$_[1]]}
+
+sub CLEAR {
+	$_[0][1]{$_[0][2]} = undef;
+	@{$_[0][0]} = ();
+}
+
+sub POP {
+	$_[0][1]{$_[0][2]} = undef;
+	pop(@{$_[0][0]});
+}
+
+sub PUSH {
+	my $o = shift;
+	$o->[1]{$o->[2]} = undef;
+	push(@{$o->[0]}, @_);
+}
+
+sub SHIFT {
+	$_[0][1]{$_[0][2]} = undef;
+	shift(@{$_[0][0]});
+}
+
+sub UNSHIFT {
+	my $o = shift;
+	$o->[1]{$o->[2]} = undef;
+	unshift(@$o, @_);
+}
+sub EXISTS {exists $_[0][0]->[$_[1]]}
+sub DELETE {delete $_[0][0]->[$_[1]]}
+
+sub SPLICE {
+	my $ob  = shift;
+	my $sz  = $ob->FETCHSIZE;
+	my $off = @_ ? shift : 0;
+	$off += $sz if $off < 0;
+	my $len = @_ ? shift : $sz - $off;
+	$ob->[1]{$ob->[2]} = undef;
+	return splice(@{$ob->[0]}, $off, $len, @_);
+}
+
+package DBIx::Struct::JSON::Hash;
+use strict;
+use warnings;
+
+sub TIEHASH {
+	bless [$_[1], $_[2], $_[3]], $_[0];
+}
+
+sub STORE {
+	$_[0][1]{$_[0][2]} = undef;
+	$_[0][0]->{$_[1]} = $_[2];
+}
+
+sub FETCH {
+	$_[0][0]->{$_[1]};
+}
+
+sub FIRSTKEY {
+	my $a = scalar keys %{$_[0][0]};
+	each %{$_[0][0]};
+}
+
+sub NEXTKEY {
+	each %{$_[0][0]};
+}
+
+sub EXISTS {
+	exists $_[0][0]->{$_[1]};
+}
+
+sub DELETE {
+	$_[0][1]{$_[0][2]} = undef;
+	delete $_[0][0]->{$_[1]};
+}
+
+sub CLEAR {
+	$_[0][1]{$_[0][2]} = undef;
+	%{$_[0][0]} = ();
+}
+
+sub SCALAR {
+	scalar %{$_[0][0]};
+}
+
+package DBIx::Struct::JSON;
+
+use strict;
+use warnings;
+use JSON;
+
+sub factory {
+	my ($class, $value_ref, $update_hash, $hash_key) = @_;
+	my $self;
+	if (not ref $$value_ref) {
+		return if not $$value_ref;
+		my $jv = from_json $$value_ref;
+		$$value_ref = $jv if $jv;
+	}
+	if ('HASH' eq ref $$value_ref) {
+		my %h;
+		tie %h, 'DBIx::Struct::JSON::Hash', $$value_ref, $update_hash, $hash_key;
+		$self = [\%h, $$value_ref];
+	} elsif ('ARRAY' eq ref $$value_ref) {
+		my @a;
+		tie @a, 'DBIx::Struct::JSON::Array', $$value_ref, $update_hash, $hash_key;
+		$self = [\@a, $$value_ref];
+	}
+	$$value_ref = bless $self, $class;
+}
+
+sub revert {
+	$_[0] = to_json $_[0][1];
+}
+
+sub accessor {
+	$_[0][0];
+}
+
 package DBIx::Struct::Connector;
 use strict;
 use warnings;
@@ -8,7 +148,7 @@ our $db_reconnect_timeout = 30;
 sub _connect {
 	my ($self, @args) = @_;
 	for my $try (1 .. $db_reconnect_timeout) {
-		my $dbh = eval { $self->SUPER::_connect(@args) };
+		my $dbh = eval {$self->SUPER::_connect(@args)};
 		return $dbh if $dbh;
 		sleep 1 if $try != $db_reconnect_timeout;
 	}
@@ -25,7 +165,7 @@ sub error_message (+%) {
 	my $msg = $_[0];
 	delete $msg->{result};
 	my $message = delete $msg->{message};
-	croak join "; ", $message, map { "$_: $msg->{$_}" } keys %$msg;
+	croak join "; ", $message, map {"$_: $msg->{$_}"} keys %$msg;
 }
 
 package DBIx::Struct::Error::Hash;
@@ -43,25 +183,27 @@ use SQL::Abstract;
 use Digest::MD5;
 use Data::Dumper;
 use base 'Exporter';
-use v5.10;
+use v5.14;
 
-our $VERSION = '0.06';
+our $VERSION = '0.09';
 
 our @EXPORT = qw{
-  one_row
-  all_rows
-  new_row
+	one_row
+	all_rows
+	new_row
 };
 
 our @EXPORT_OK = qw{
-  connector
-  hash_ref_slice
+	connector
+	hash_ref_slice
 };
 
 our $conn;
 our $update_on_destroy     = 1;
 our $connector_module      = 'DBIx::Struct::Connector';
 our $connector_constructor = 'new';
+our $connector_pool;
+our $connector_pool_method = 'get_connector';
 our $connector_args        = [];
 our $connector_driver;
 our $table_classes_namespace = 'DBC';
@@ -135,9 +277,39 @@ sub hash_ref_slice($@) {
 	error_message {
 		message => "first parameter is not hash reference",
 		result  => 'INTERR',
-	  }
-	  if 'HASH' ne ref $hashref;
-	map { $_ => $hashref->{$_} } @slice;
+		}
+		if 'HASH' ne ref $hashref;
+	map {$_ => $hashref->{$_}} @slice;
+}
+
+my @already_exported_to;
+
+sub connector {
+	$conn;
+}
+
+sub connector_from_pool {
+	$connector_pool->$connector_pool_method();
+}
+
+sub set_connector_pool {
+	$connector_pool = $_[0];
+	if (\&connector != \&connector_from_pool) {
+		no warnings 'redefine';
+		no strict 'refs';
+		*connector = \&connector_from_pool;
+		for my $aep (@already_exported_to) {
+			*{"$aep\::connector"} = \&connector;
+		}
+	}
+}
+
+sub set_connector_pool_method {
+	$connector_pool_method = $_[0];
+}
+
+sub set_connector_object {
+	*conn = \$_[0];
 }
 
 sub check_package_scalar {
@@ -151,9 +323,11 @@ sub check_package_scalar {
 
 sub import {
 	my ($class, @args) = @_;
-	state $init_import = 0;
 	my $defconn = 0;
-	for (my $i = 0 ; $i < @args ; ++$i) {
+	my $_emc    = 0;
+	my $_cp     = 0;
+	my $_c      = 0;
+	for (my $i = 0; $i < @args; ++$i) {
 		if ($args[$i] eq 'connector_module') {
 			(undef, $connector_module) = splice @args, $i, 2;
 			--$i;
@@ -175,33 +349,48 @@ sub import {
 			--$i;
 		} elsif ($args[$i] eq 'error_class') {
 			my (undef, $emc) = splice @args, $i, 2;
-			$error_message_class = $emc if !$init_import;
+			$error_message_class = $emc;
+			$_emc                = 1;
+			--$i;
+		} elsif ($args[$i] eq 'connector_pool') {
+			(undef, $connector_pool) = splice @args, $i, 2;
+			$_cp = 1;
+			--$i;
+		} elsif ($args[$i] eq 'connector_pool_method') {
+			(undef, $connector_pool_method) = splice @args, $i, 2;
 			--$i;
 		} elsif ($args[$i] eq 'connector_args') {
 			(undef, $connector_args) = splice @args, $i, 2;
 			--$i;
+		} elsif ($args[$i] eq 'connector') {
+			$_c = 1;
 		} elsif ($args[$i] eq 'connector_object') {
 			$defconn = 1;
 			my (undef, $connector_object) = splice @args, $i, 2;
 			--$i;
-			*conn = \${$connector_object};
+			*conn = \$connector_object;
 		}
 	}
-	if (!$init_import) {
+	if ($_emc) {
 		my $eval = "*error_message = \\&$error_message_class" . "::error_message";
 		eval $eval;
 	}
-	my %imps = map { $_ => undef } @args, @EXPORT;
+	if ($_cp) {
+		no warnings 'redefine';
+		no strict 'refs';
+		*connector = \&connector_from_pool;
+		for my $aep (@already_exported_to) {
+			*{"$aep\::connector"} = \&connector;
+		}
+	}
+	my $callpkg = caller;
+	push @already_exported_to, $callpkg if $_c;
+	my %imps = map {$_ => undef} @args, @EXPORT;
 	$class->export_to_level(1, $class, keys %imps);
-	$init_import = 1;
-}
-
-sub connector {
-	$conn;
 }
 
 sub _not_yet_connected {
-	if (not $conn) {
+	if (!$connector_pool && !$conn) {
 		my ($dsn, $user, $password) = @_;
 		if ($dsn && $dsn !~ /^dbi:/i) {
 			$dsn = "dbi:Pg:dbname=$dsn";
@@ -226,18 +415,18 @@ sub _not_yet_connected {
 			@$connector_args = ($dsn, $user, $password, $connect_attrs);
 		}
 		$conn = $connector_module->$connector_constructor(@$connector_args)
-		  or error_message {
+			or error_message {
 			message => "DB connect error",
 			result  => 'SQLERR',
-		  };
+			};
 		$conn->mode('fixup');
 	}
 	'' =~ /()/;
-	$connector_driver = $conn->driver->{driver};
+	$connector_driver = connector->driver->{driver};
 	no warnings 'redefine';
 	*connect = \&connector;
 	populate();
-	$conn;
+	connector;
 }
 
 sub connect {
@@ -249,10 +438,10 @@ sub connect {
 
 	sub make_name {
 		my ($table) = @_;
-		my $simple_table = (index ($table, " ") == -1);
+		my $simple_table = (index($table, " ") == -1);
 		my $ncn;
 		if ($simple_table) {
-			$ncn = $table_classes_namespace . "::" . join ('', map { ucfirst ($_) } split (/[^a-zA-Z0-9]/, $table));
+			$ncn = $table_classes_namespace . "::" . join('', map {ucfirst($_)} split(/[^a-zA-Z0-9]/, $table));
 		} else {
 			$md5->add($table);
 			$ncn = $query_classes_namespace . "::" . "G" . $md5->hexdigest;
@@ -269,15 +458,14 @@ sub populate {
 			my $sth = $_->table_info('', '', '%', "TABLE");
 			return if not $sth;
 			my $tables = $sth->fetchall_arrayref;
-			@tables =
-			  map { $_->[2] } grep { $_->[3] eq 'TABLE' and $_->[2] !~ /^sql_/ } @$tables;
+			@tables = map {$_->[2]} grep {$_->[3] eq 'TABLE' and $_->[2] !~ /^sql_/} @$tables;
 		}
 	);
 	setup_row($_) for @tables;
 }
 
-sub _row_data ()    { 0 }
-sub _row_updates () { 1 }
+sub _row_data ()    {0}
+sub _row_updates () {1}
 
 sub make_object_new {
 	my ($table, $required, $pk_row_data, $pk_returninig) = @_;
@@ -285,18 +473,20 @@ sub make_object_new {
 		sub new {
 			my \$class = \$_[0];
 			my \$self = [ [] ];
+			bless \$self, \$class;
 			if(CORE::defined(\$_[1]) && CORE::ref(\$_[1]) eq 'ARRAY') {
 				\$self->[@{[_row_data]}] = \$_[1];
 			}
 NEW
 	if (not ref $table) {
 		$new .= <<NEW;
-			 elsif(CORE::defined \$_[1]) {
+			 else {
 				my \%insert;
 				for(my \$i = 1; \$i < \@_; \$i += 2) {
 					if (CORE::exists \$fields{\$_[\$i]}) {
+						my \$f = \$_[\$i];
 						\$self->[@{[_row_data]}]->[\$fields{\$_[\$i]}] = \$_[\$i + 1];
-						\$insert{\$_[\$i]} = \$_[\$i + 1];
+						\$insert{\$f} = \$_[\$i + 1];
 					} else {
 						DBIx::Struct::error_message {
 							result  => 'SQLERR',
@@ -323,14 +513,24 @@ NEW
 						} elsif(CORE::ref(\$insert{\$_}) eq 'SCALAR') {
 							CORE::push \@values, \${\$insert{\$_}};
 							"\$_";
+						} elsif(CORE::exists (\$json_fields{\$_})
+							&& (CORE::ref(\$insert{\$_}) eq 'ARRAY' || CORE::ref(\$insert{\$_}) eq 'HASH')) {
+							CORE::push \@bind, JSON::to_json(\$insert{\$_});
+							CORE::push \@values, "?";
+							"\$_" 
 						} else {
 							CORE::push \@bind, \$insert{\$_};
 							CORE::push \@values, "?";
 							"\$_" 
 						}						
 					} CORE::keys \%insert;
-				my \$insert = "insert into $table (" . CORE::join( ", ", \@insert) . ") values ("
+				my \$insert;
+				if(\%insert){
+					\$insert = "insert into $table (" . CORE::join( ", ", \@insert) . ") values ("
 					.  CORE::join( ", ", \@values) . ")";
+				} else {
+					\$insert = "insert into $table values (default)";
+				}
 NEW
 		if ($required) {
 			$new .= <<NEW;
@@ -353,7 +553,7 @@ NEW
 NEW
 	}
 	$new .= <<NEW;
-	  		bless \$self, \$class;
+	  		\$self;
 		}
 NEW
 	$new;
@@ -366,11 +566,17 @@ sub make_object_filter_timestamp {
 			my \$self = \$_[0];
 			if(\@_ == 1) {
 				for my \$f ($timestamps) {
-					\$self->[@{[_row_data]}][\$fields{\$f}] =~ s/\\.\\d+\$// if \$self->[@{[_row_data]}][\$fields{\$f}];
+					if(\$self->[@{[_row_data]}][\$fields{\$f}]) {
+						\$self->[@{[_row_data]}][\$fields{\$f}] =~ s/\\.\\d+(\$|\\+|\\-)/$1/;
+						\$self->[@{[_row_data]}][\$fields{\$f}] =~ s/(\\+|\\-)(\\d{2})\$/\$1\${2}00/;
+					}
 				}
 			} else {
 				for my \$f (\@_[1..\$#_]) {
-					\$self->[@{[_row_data]}][\$fields{\$f}] =~ s/\\.\\d+\$// if \$self->[@{[_row_data]}][\$fields{\$f}];
+					if(\$self->[@{[_row_data]}][\$fields{\$f}]) {
+						\$self->[@{[_row_data]}][\$fields{\$f}] =~ s/\\.\\d+(\$|\\+|\\-)/$1/;
+						\$self->[@{[_row_data]}][\$fields{\$f}] =~ s/(\\+|\\-)(\\d{2})\$/\$1\${2}00/;
+					}
 				}
 			}
 			'' =~ /()/;
@@ -390,16 +596,15 @@ sub make_object_set {
 					\$self->[@{[_row_updates]}] = {};
 				} elsif(CORE::ref(\$_[1]) eq 'HASH') {
 					for my \$f (CORE::keys \%{\$_[1]}) {
-						if (CORE::exists \$fields{\$_[\$f]}) {
-							\$self->[@{[_row_data]}]->[\$fields{\$f}] = \$_[1]->{\$f};
-							\$self->[@{[_row_updates]}]{\$f} = undef;
+						if (CORE::exists \$fields{\$f}) {
+							\$self->\$f(\$_[1]->{\$f});
 						}
 					}
 				} elsif(not CORE::ref(\$_[1])) {
 					for(my \$i = 1; \$i < \@_; \$i += 2) {
 						if (CORE::exists \$fields{\$_[\$i]}) {
-							\$self->[@{[_row_data]}]->[\$fields{\$_[\$i]}] = \$_[\$i + 1];
-							\$self->[@{[_row_updates]}]{\$_[\$i]} = undef;
+							my \$f = \$fields{\$_[\$i]};
+							\$self->\$f(\$_[\$i + 1]);
 						}
 					}
 				}
@@ -431,7 +636,13 @@ sub make_object_data {
 			} else {
 				\@ret_keys = keys \%fields;
 			}
-			\$ret = { CORE::map {\$_ => \$self->[@{[_row_data]}]->[\$fields{\$_}] } \@ret_keys} if not CORE::defined \$ret;
+			\$ret = { 
+				CORE::map { 
+					CORE::exists(\$json_fields{\$_})
+					? (\$_ => \$self->\$_->accessor)
+					: (\$_ => \$self->\$_) 
+				} \@ret_keys
+			} if not CORE::defined \$ret;
 			\$ret;
 		}
 DATA
@@ -442,6 +653,7 @@ sub make_object_update {
 	my ($table, $pk_where, $pk_row_data) = @_;
 	my $update;
 	if (not ref $table) {
+
 		# means this is just one simple table
 		$update = <<UPD;
 		sub update {
@@ -465,13 +677,16 @@ sub make_object_update {
 							} elsif(CORE::ref(\$set_hash{\$_}) eq 'REF' and CORE::ref(\${\$set_hash{\$_}}) eq 'ARRAY') {
 								if(CORE::defined \${\$set_hash{\$_}}->[0]) {
 									CORE::push \@bind, \@{\${\$set_hash{\$_}}}[1..\$#{\${\$set_hash{\$_}}}];
-									CORE::push \@values, \${\$set_hash{\$_}}->[0];
 									"\$_ = " . \${\$set_hash{\$_}}->[0];
 								} else {
 									"\$_ = null"
 								}
 							} elsif(CORE::ref(\$set_hash{\$_}) eq 'SCALAR') {
 								"\$_ = " . \${\$set_hash{\$_}};
+							} elsif(CORE::exists(\$json_fields{\$_})
+								&& (CORE::ref(\$set_hash{\$_}) eq 'ARRAY' || CORE::ref(\$set_hash{\$_}) eq 'HASH')) {
+								CORE::push \@bind, JSON::to_json(\$set_hash{\$_});
+								"\$_ = ?" 
 							} else {
 								CORE::push \@bind, \$set_hash{\$_};
 								"\$_ = ?" 
@@ -485,7 +700,7 @@ sub make_object_update {
 					}
 					(\$where, \@bind_where) = SQL::Abstract->new->where(\$cond);
 				}
-				DBIx::Struct::connect->run(sub {
+				return DBIx::Struct::connect->run(sub {
 					\$_->do(qq{update $table set \$set \$where}, undef, \@bind, \@bind_where)
 					or DBIx::Struct::error_message {
 						result  => 'SQLERR',
@@ -510,6 +725,10 @@ sub make_object_update {
 								} else {
 									"\$_ = null"
 								}
+							} elsif(CORE::ref(\$column_value) && CORE::ref(\$column_value) =~ /^DBIx::Struct::JSON/) {
+								\$column_value->revert;
+								CORE::push \@bind, \$column_value;
+								"\$_ = ?" 
 							} elsif(CORE::ref(\$column_value) eq 'SCALAR') {
 								"\$_ = " . \$\$column_value;
 							} else {
@@ -557,7 +776,7 @@ sub make_object_delete {
 					\$cond = {(selectKeys)[0] => \$_[1]};
 				}
 				(\$where, \@bind) = SQL::Abstract->new->where(\$cond);
-				DBIx::Struct::connect->run(sub {
+				return DBIx::Struct::connect->run(sub {
 					\$_->do(qq{delete from $table \$where}, undef, \@bind)
 					or DBIx::Struct::error_message {
 						result  => 'SQLERR',
@@ -632,7 +851,7 @@ FETCH
 sub _exists_row ($) {
 	my $ncn = $_[0];
 	no strict "refs";
-	if (grep { !/::$/ } keys %{"${ncn}::"}) {
+	if (grep {!/::$/} keys %{"${ncn}::"}) {
 		return 1;
 	}
 	return;
@@ -648,8 +867,8 @@ sub _parse_interface ($) {
 			error_message {
 				result  => 'SQLERR',
 				message => "Unknown base interface table $i",
-			  }
-			  unless _exists_row $dbc_name;
+				}
+				unless _exists_row $dbc_name;
 			no strict 'refs';
 			my $href = \%{"${dbc_name}::fkfuncs"};
 			if ($href && %$href) {
@@ -663,14 +882,15 @@ sub _parse_interface ($) {
 			error_message {
 				result  => 'SQLERR',
 				message => "Unknown base interface table $i",
-			  }
-			  unless _exists_row $dbc_name;
+				}
+				unless _exists_row $dbc_name;
 			no strict 'refs';
 			my $href = \%{"${dbc_name}::fkfuncs"};
 			next if not $href or not %$href;
 			my $fl = $interface->{$i};
 			$fl = [$fl] if not ref $fl;
 			if ('ARRAY' eq ref $fl) {
+
 				for my $m (@$fl) {
 					$ret{$m} = $href->{$m} if exists $href->{$m};
 				}
@@ -684,7 +904,7 @@ sub _parse_interface ($) {
 	} else {
 		error_message {
 			result  => 'SQLERR',
-			message => "Unknown interface structure: " . ref ($interface),
+			message => "Unknown interface structure: " . ref($interface),
 		};
 	}
 	\%ret;
@@ -692,12 +912,11 @@ sub _parse_interface ($) {
 
 sub setup_row {
 	my ($table, $ncn, $interface) = @_;
-	my $conn = DBIx::Struct::connect;
 	error_message {
 		result  => 'SQLERR',
 		message => "Unsupported driver $connector_driver",
-	  }
-	  unless exists $driver_pk_insert{$connector_driver};
+		}
+		unless exists $driver_pk_insert{$connector_driver};
 	$ncn ||= make_name($table);
 	return $ncn if _exists_row $ncn ;
 	my %fields;
@@ -707,16 +926,19 @@ sub setup_row {
 	my @pkeys;
 	my @fkeys;
 	my @refkeys;
+	my %json_fields;
+	my $connector = DBIx::Struct::connect;
+
 	if (not ref $table) {
 		# means this is just one simple table
-		$conn->run(
+		$connector->run(
 			sub {
 				my $cih = $_->column_info(undef, undef, $table, undef);
 				error_message {
 					result  => 'SQLERR',
 					message => "Unknown table $table",
-				  }
-				  if not $cih;
+					}
+					if not $cih;
 				my $i = 0;
 				while (my $chr = $cih->fetchrow_hashref) {
 					$chr->{COLUMN_NAME} =~ s/"//g;
@@ -724,7 +946,10 @@ sub setup_row {
 					if ($chr->{TYPE_NAME} =~ /^time/) {
 						push @timestamp_fields, $chr->{COLUMN_NAME};
 					}
-					if ($chr->{NULLABLE} == 0 && !defined ($chr->{COLUMN_DEF})) {
+					if ($chr->{TYPE_NAME} =~ /^json/) {
+						$json_fields{$chr->{COLUMN_NAME}} = undef;
+					}
+					if ($chr->{NULLABLE} == 0 && !defined($chr->{COLUMN_DEF})) {
 						push @required, $chr->{COLUMN_NAME};
 					}
 					$fields{$chr->{COLUMN_NAME}} = $i++;
@@ -732,7 +957,7 @@ sub setup_row {
 				@pkeys = $_->primary_key(undef, undef, $table);
 				if (!@pkeys && @required) {
 					my $ukh = $_->statistics_info(undef, undef, $table, 1, 1);
-					my %req = map { $_ => undef } @required;
+					my %req = map {$_ => undef} @required;
 					my %pkeys;
 					while (my $ukr = $ukh->fetchrow_hashref) {
 						if (not exists $req{$ukr->{COLUMN_NAME}} or defined $ukr->{FILTER_CONDITION}) {
@@ -741,47 +966,49 @@ sub setup_row {
 							$pkeys{$ukr->{INDEX_NAME}}{fields}{$ukr->{COLUMN_NAME}} = undef;
 						}
 					}
-					my @d = grep { exists $pkeys{$_}{drop} } keys %pkeys;
+					my @d = grep {exists $pkeys{$_}{drop}} keys %pkeys;
 					delete $pkeys{$_} for @d;
 					if (%pkeys) {
-						my @spk = sort { scalar (keys %{$pkeys{$a}{fields}}) <=> scalar (keys %{$pkeys{$b}{fields}}) }
-						  keys %pkeys;
+						my @spk = sort {scalar(keys %{$pkeys{$a}{fields}}) <=> scalar(keys %{$pkeys{$b}{fields}})}
+							keys %pkeys;
 						@pkeys = keys %{$pkeys{$spk[0]}{fields}};
 					}
 				}
 				my $sth = $_->foreign_key_info(undef, undef, undef, undef, undef, $table);
 				if ($sth) {
-					@fkeys =
-					  grep { $_->{FK_COLUMN_NAME} !~ /[^a-z_0-9]/ } @{$sth->fetchall_arrayref({})};
+					@fkeys = grep {$_->{FK_COLUMN_NAME} !~ /[^a-z_0-9]/} @{$sth->fetchall_arrayref({})};
 				}
 				$sth = $_->foreign_key_info(undef, undef, $table, undef, undef, undef);
 				if ($sth) {
-					@refkeys =
-					  grep { $_->{FK_COLUMN_NAME} !~ /[^a-z_0-9]/ } @{$sth->fetchall_arrayref({})};
+					@refkeys = grep {$_->{FK_COLUMN_NAME} !~ /[^a-z_0-9]/} @{$sth->fetchall_arrayref({})};
 				}
 			}
 		);
 	} else {
+
 		# means this is a query
 		%fields = %{$table->{NAME_hash}};
-		$conn->run(
+		$connector->run(
 			sub {
-				for (my $cn = 0 ; $cn < @{$table->{NAME}} ; ++$cn) {
+				for (my $cn = 0; $cn < @{$table->{NAME}}; ++$cn) {
 					my $ti = $_->type_info($table->{TYPE}->[$cn]);
 					push @timestamp_fields, $table->{NAME}->[$cn]
-					  if $ti && $ti->{TYPE_NAME} =~ /^time/;
+						if $ti && $ti->{TYPE_NAME} =~ /^time/;
+					$json_fields{$table->{NAME}->[$cn]} = undef
+						if $ti && $ti->{TYPE_NAME} =~ /^json/;
 				}
 			}
 		);
 	}
-	my $fields = join ", ", map { qq|"$_" => $fields{$_}| } keys %fields;
-	my $required = '';
+	my $fields      = join ", ", map {qq|"$_" => $fields{$_}|} keys %fields;
+	my $json_fields = join ", ", map {qq|"$_" => undef|} keys %json_fields;
+	my $required    = '';
 	if (@required) {
-		$required = join (", ", map { qq|"$_"| } @required);
+		$required = join(", ", map {qq|"$_"|} @required);
 	}
 	my $timestamps = '';
 	if (@timestamp_fields) {
-		$timestamps = join (", ", map { qq|"$_"| } @timestamp_fields);
+		$timestamps = join(", ", map {qq|"$_"|} @timestamp_fields);
 	} else {
 		$timestamps = "()";
 	}
@@ -802,10 +1029,10 @@ sub setup_row {
 	my %pk_fields;
 	if (@pkeys) {
 		@pk_fields{@pkeys} = undef;
-		$pk_row_data = join (", ", map { qq|\$self->[@{[_row_data]}]->[$fields{"$_"}]| } @pkeys);
-		$pk_returninig = 'returning ' . join (", ", @pkeys);
-		$pk_where = join (" and ", map { "$_ = ?" } @pkeys);
-		my $sk_list = join (", ", map { qq|"$_"| } @pkeys);
+		$pk_row_data = join(", ", map {qq|\$self->[@{[_row_data]}]->[$fields{"$_"}]|} @pkeys);
+		$pk_returninig = 'returning ' . join(", ", @pkeys);
+		$pk_where = join(" and ", map {"$_ = ?"} @pkeys);
+		my $sk_list = join(", ", map {qq|"$_"|} @pkeys);
 		$select_keys = <<SK;
 		sub selectKeys () { 
 		 	($sk_list) 
@@ -813,7 +1040,7 @@ sub setup_row {
 SK
 	} else {
 		if (@fields) {
-			my $sk_list = join (", ", map { qq|"$_"| } @fields);
+			my $sk_list = join(", ", map {qq|"$_"|} @fields);
 			$select_keys = <<SK;
 		sub selectKeys () { 
 			($sk_list)
@@ -903,13 +1130,19 @@ FKT
 		}
 RT
 	}
-	my $accessors = '';
+	my $accessors = <<ACC;
+		sub markUpdated {
+			\$_[0]->[@{[_row_updates]}]{\$_[1]} = undef if CORE::exists \$fields{\$_[1]};
+			\$_[0];
+		}
+ACC
 	for my $k (keys %fields) {
 		next if exists $keywords{$k};
 		next if $k =~ /^\d/;
 		$k =~ s/[^\w\d]/_/g;
-		if (!exists ($pk_fields{$k}) && (not ref $table)) {
-			$accessors .= <<ACC;
+		if (!exists $json_fields{$k}) {
+			if (!exists($pk_fields{$k}) && (not ref $table)) {
+				$accessors .= <<ACC;
 		sub $k {
 			if(\@_ > 1) {
 				\$_[0]->[@{[_row_data]}]->[$fields{$k}] = \$_[1];
@@ -918,14 +1151,44 @@ RT
 			\$_[0]->[@{[_row_data]}]->[$fields{$k}];
 		}
 ACC
-		} else {
-			$accessors .= <<ACC;
+			} else {
+				$accessors .= <<ACC;
 		sub $k {
 			\$_[0]->[@{[_row_data]}]->[$fields{$k}];
 		}
 ACC
+			}
+		} else {
+			if (!exists($pk_fields{$k}) && (not ref $table)) {
+				$accessors .= <<ACC;
+		sub $k {
+			if(\@_ > 1) {
+				if(not CORE::ref \$_[1]) {
+					\$_[0]->[@{[_row_data]}]->[$fields{$k}] = \$_[1];
+				} else {
+					\$_[0]->[@{[_row_data]}]->[$fields{$k}] = JSON::to_json(\$_[1]);
+				}
+				\$_[0]->[@{[_row_updates]}]{"$k"} = undef;
+			}
+			if(not CORE::ref \$_[0]->[@{[_row_data]}]->[$fields{$k}]) {
+				\$_[0]->[@{[_row_updates]}] = {} if not \$_[0]->[@{[_row_updates]}];
+				\$_[0]->[@{[_row_data]}]->[$fields{$k}] = 
+					DBIx::Struct::JSON->factory(\\\$_[0]->[@{[_row_data]}]->[$fields{$k}], \$_[0]->[@{[_row_updates]}], "$k");
+			}
+			\$_[0]->[@{[_row_data]}]->[$fields{$k}]->accessor;
+		}
+ACC
+			} else {
+				$accessors .= <<ACC;
+		sub $k {
+			if(\$_[0]->[@{[_row_data]}]->[$fields{$k}] and not CORE::ref \$_[0]->[@{[_row_data]}]->[$fields{$k}]) {
+				\$_[0]->[@{[_row_data]}]->[$fields{$k}] = JSON::from_json(\$_[0]->[@{[_row_data]}]->[$fields{$k}]);
+			}
+			\$_[0]->[@{[_row_data]}]->[$fields{$k}];
+		}
+ACC
+			}
 		}
-
 	}
 	my $package_header = <<PHD;
 		package ${ncn};
@@ -933,11 +1196,13 @@ ACC
 		use warnings;
 		use Carp;
 		use SQL::Abstract;
+		use JSON;
 		our \%fields = ($fields);
+		our \%json_fields = ($json_fields);
 PHD
 	if (not ref $table) {
 		if (%fkfuncs) {
-			my $fkfuncs = join ",", map { qq{"$_" => \\&${ncn}::$_} } keys %fkfuncs;
+			my $fkfuncs = join ",", map {qq{"$_" => \\&${ncn}::$_}} keys %fkfuncs;
 			$package_header .= <<PHD;
 		our \%fkfuncs = ($fkfuncs);
 PHD
@@ -958,6 +1223,7 @@ PHD
 	my $delete           = make_object_delete($table, $pk_where, $pk_row_data);
 	my $fetch            = make_object_fetch($table, $pk_where, $pk_row_data);
 	my $destroy;
+
 	if (not ref $table) {
 		$destroy = <<DESTROY;
 		sub DESTROY {
@@ -969,10 +1235,11 @@ DESTROY
 		$destroy = '';
 	}
 	my $eval_code = join "", $package_header, $select_keys, $new,
-	  $filter_timestamp, $set, $data, $fetch,
-	  $update, $delete, $destroy, $accessors, $foreign_tables,
-	  $references_tables;
-	#print $eval_code;
+		$filter_timestamp, $set, $data, $fetch,
+		$update, $delete, $destroy, $accessors, $foreign_tables,
+		$references_tables;
+
+	# print $eval_code;
 	eval $eval_code;
 	error_message {
 		result  => 'SQLERR',
@@ -989,10 +1256,10 @@ DESTROY
 	return $ncn;
 }
 
-sub _table_name()    { 0 }
-sub _table_alias()   { 1 }
-sub _table_join()    { 2 }
-sub _table_join_on() { 3 }
+sub _table_name()    {0}
+sub _table_alias()   {1}
+sub _table_join()    {2}
+sub _table_join_on() {3}
 
 my $sql_abstract = SQL::Abstract->new;
 my $tblnum;
@@ -1003,15 +1270,15 @@ sub _build_complex_query {
 	my @from;
 	my @columns;
 	my @linked_list = (
-		ref ($table) eq 'ARRAY'
+		ref($table) eq 'ARRAY'
 		? @$table
 		: error_message {
 			result  => 'SQLERR',
-			message => "Unsupported type of query: " . ref ($table)
+			message => "Unsupported type of query: " . ref($table)
 		}
 	);
 	my ($conditions, $groupby, $having, $limit, $offset);
-	for (my $i = 0 ; $i < @linked_list ; ++$i) {
+	for (my $i = 0; $i < @linked_list; ++$i) {
 		my $le = $linked_list[$i];
 		if ('ARRAY' eq ref $le) {
 			my $subfrom = _build_complex_query($le, $query_bind);
@@ -1020,17 +1287,17 @@ sub _build_complex_query {
 			push @from, ["($subfrom)", $ta];
 			next;
 		}
-		if (substr ($le, 0, 1) ne '-') {
+		if (substr($le, 0, 1) ne '-') {
 			my ($tn, $ta) = split ' ', $le;
 			$ta = $tn if not $ta;
 			error_message {
 				result  => 'SQLERR',
 				message => "Unknown table $tn"
-			  }
-			  unless _exists_row(make_name($tn));
+				}
+				unless _exists_row(make_name($tn));
 			push @from, [$tn, $ta];
 		} else {
-			my $cmd = substr ($le, 1);
+			my $cmd = substr($le, 1);
 			if ($cmd eq 'left') {
 				$from[-1][_table_join] = 'left join';
 			} elsif ($cmd eq 'right') {
@@ -1055,7 +1322,7 @@ sub _build_complex_query {
 				$offset = 0 + $linked_list[++$i];
 			} elsif ($cmd eq 'columns') {
 				my $cols = $linked_list[++$i];
-				if (ref ($cols)) {
+				if (ref($cols)) {
 					push @columns, @$cols;
 				} else {
 					push @columns, $cols;
@@ -1067,17 +1334,17 @@ sub _build_complex_query {
 		result  => 'SQLERR',
 		message => "No table to build query on"
 	} if !@from;
-	for (my $idx = 1 ; $idx < @from ; ++$idx) {
+	for (my $idx = 1; $idx < @from; ++$idx) {
 		next if $from[$idx][_table_join_on] or not $from[$idx - 1][_table_join];
-		next if substr ($from[$idx][_table_name], 0, 1) eq "(";
+		next if substr($from[$idx][_table_name], 0, 1) eq "(";
 		my $cta  = $from[$idx][_table_alias];
 		my $cto  = make_name($from[$idx][_table_name]);
 		my $ucct = ucfirst $from[$idx][_table_name];
 		my @join;
-		for (my $i = $idx - 1 ; $i >= 0 ; --$i) {
+		for (my $i = $idx - 1; $i >= 0; --$i) {
 			next if not $from[$i][_table_join];
 			my $ptn = $from[$i][_table_name];
-			next if substr ($ptn, 0, 1) eq "(";
+			next if substr($ptn, 0, 1) eq "(";
 			my $ucfptn = ucfirst $ptn;
 			if ($cto->can("foreignKey$ucfptn")) {
 				my $fkfn = "foreignKey$ucfptn";
@@ -1092,12 +1359,12 @@ sub _build_complex_query {
 				}
 			}
 		}
-		$from[$idx][_table_join_on] = ["on", join (" and ", @join)];
+		$from[$idx][_table_join_on] = ["on", join(" and ", @join)];
 	}
 	my $from = '';
 	@columns = ('*') if not @columns;
 	my $joined = 0;
-	for (my $idx = 0 ; $idx < @from ; ++$idx) {
+	for (my $idx = 0; $idx < @from; ++$idx) {
 		if (not $joined) {
 			$from .= " " . $from[$idx][_table_name];
 			$from .= " " . $from[$idx][_table_alias] if $from[$idx][_table_alias] ne $from[$idx][_table_name];
@@ -1122,7 +1389,7 @@ sub _build_complex_query {
 			$joined = 0;
 		}
 	}
-	my $ret = "select " . join (", ", @columns) . " from" . $from;
+	my $ret = "select " . join(", ", @columns) . " from" . $from;
 	if (not defined $where) {
 		my $sql_grp     = _parse_groupby($groupby);
 		my $having_bind = [];
@@ -1154,8 +1421,8 @@ sub _parse_groupby {
 	my $sql_grp;
 	if (defined $groupby) {
 		$sql_grp = "GROUP BY ";
-		my @groupby = map { /^\d+$/ ? $_ : qq{"$_"} } (ref ($groupby) ? @$groupby : ($groupby));
-		$sql_grp .= join (", ", @groupby);
+		my @groupby = map {/^\d+$/ ? $_ : "$_"} (ref($groupby) ? @$groupby : ($groupby));
+		$sql_grp .= join(", ", @groupby);
 	}
 	$sql_grp;
 }
@@ -1172,8 +1439,8 @@ sub _parse_having {
 }
 
 sub execute {
-	my ($groupby, $having, $up_conditions, $up_order, $up_limit, $up_offset, $up_interface);
-	for (my $i = 2 ; $i < @_ ; ++$i) {
+	my ($groupby, $having, $up_conditions, $up_order, $up_limit, $up_offset, $up_interface, $sql, $dry_run);
+	for (my $i = 2; $i < @_; ++$i) {
 		next unless defined $_[$i] and not ref $_[$i];
 		if ($_[$i] eq '-group_by') {
 			(undef, $groupby) = splice @_, $i, 2;
@@ -1196,6 +1463,12 @@ sub execute {
 		} elsif ($_[$i] eq '-offset') {
 			(undef, $up_offset) = splice @_, $i, 2;
 			--$i;
+		} elsif ($_[$i] eq '-sql') {
+			(undef, $sql) = splice @_, $i, 2;
+			--$i;
+		} elsif ($_[$i] eq '-dry_run') {
+			(undef, $dry_run) = splice @_, $i, 2;
+			--$i;
 		}
 	}
 	$tblnum = 1;
@@ -1215,18 +1488,19 @@ sub execute {
 	my $where;
 	my $need_where = 0;
 	my @where_bind;
-	my $simple_table = (not ref $table and index ($table, " ") == -1);
+	my $simple_table = (not ref $table and index($table, " ") == -1);
 	my $ncn;
+
 	if ($simple_table) {
 		$ncn = make_name($table);
 		setup_row($table);
 		if ($have_conditions and not ref $conditions) {
 			my $id = ($ncn->selectKeys())[0]
-			  or error_message {
+				or error_message {
 				result  => 'SQLERR',
 				message => "unknown primary key",
 				query   => "select * from $table",
-			  };
+				};
 			if (defined $conditions) {
 				$where      = "where $id = ?";
 				@where_bind = ($conditions);
@@ -1251,11 +1525,11 @@ sub execute {
 		$where .= " $order_sql";
 		push @where_bind, @order_bind;
 	}
-	if (defined ($limit)) {
+	if (defined($limit)) {
 		$limit += 0;
 		$where .= " limit $limit";
 	}
-	if (defined ($offset)) {
+	if (defined($offset)) {
 		$offset += 0;
 		$where .= " offset $offset" if $offset;
 	}
@@ -1267,35 +1541,34 @@ sub execute {
 		$query = (not ref $table) ? qq{$table $where} : _build_complex_query($table, \@query_bind, $where);
 		$ncn = make_name($query);
 	}
+	if ($sql) {
+		if ('CODE' eq ref $sql) {
+			$sql->($query, \@where_bind);
+		} elsif ('SCALAR' eq ref $sql) {
+			$$sql = $query;
+		}
+	}
+	return if $dry_run;
 	'' =~ /()/;
 	my $sth;
 	return DBIx::Struct::connect->run(
 		sub {
 			$sth = $_->prepare($query)
-			  or error_message {
+				or error_message {
 				result  => 'SQLERR',
 				message => $_->errstr,
 				query   => $query,
-			  };
+				};
 			$sth->execute(@query_bind, @where_bind)
-			  or error_message {
+				or error_message {
 				result     => 'SQLERR',
 				message    => $_->errstr,
 				query      => $query,
 				where_bind => Dumper(\@where_bind),
 				query_bind => Dumper(\@query_bind),
 				conditions => Dumper($conditions),
-			  };
+				};
 			setup_row($sth, $ncn, $up_interface);
-			no strict 'refs';
-			if (!"$ncn"->can("dumpSQL")) {
-				*{$ncn . "::dumpSQL"} = sub { $query };
-				if (@query_bind) {
-					*{$ncn . "::needQueryBind"} = sub { 1 };
-				} else {
-					*{$ncn . "::needQueryBind"} = sub { 0 };
-				}
-			}
 			return $code->($sth, $ncn);
 		}
 	);
@@ -1316,8 +1589,8 @@ sub one_row {
 
 sub all_rows {
 	my $mapfunc;
-	for (my $i = 0 ; $i < @_ ; ++$i) {
-		if (ref ($_[$i]) eq 'CODE') {
+	for (my $i = 0; $i < @_; ++$i) {
+		if (ref($_[$i]) eq 'CODE') {
 			$mapfunc = splice @_, $i, 1;
 			last;
 		}
@@ -1343,12 +1616,12 @@ sub all_rows {
 
 sub new_row {
 	my ($table, @data) = @_;
-	my $simple_table = (index ($table, " ") == -1);
+	my $simple_table = (index($table, " ") == -1);
 	error_message {
 		result  => 'SQLERR',
 		message => "insert row can't work for queries"
-	  }
-	  unless $simple_table;
+		}
+		unless $simple_table;
 	my $ncn = setup_row($table);
 	return $ncn->new(@data);
 }
